﻿using System;
using System.Windows;
using System.Net;
using System.Xml;
using System.Diagnostics;
using System.Reflection;
using System.IO;
using System.Configuration;

namespace OpenSourceControls
{
    /// <summary>
    /// AutoUpdate Dialog
    /// </summary>
    public partial class AutoUpdateDialog : Window
    {
        /// <summary>
        /// The value of the "Always check for program updates" checkbox
        /// </summary>
        public bool CheckForProgramUpdates
        {
            get { return (bool)GetValue(CheckForUpdatesProperty); }
            set { SetValue(CheckForUpdatesProperty, value); }
        }

        public static readonly DependencyProperty CheckForUpdatesProperty =
            DependencyProperty.Register("CheckForProgramUpdates", typeof(bool), typeof(AutoUpdateDialog), new UIPropertyMetadata(true));

        public bool IsUpdateAvailable { get; private set; }
        public bool IsUpdating { get; private set; }

        public string UpdateUrl { get; set; }

        public string ProductName { get; set; }
        public string DownloadUrl { get; set; }

        public Assembly CurrentAssembly { get; set; }

        public Version YourVersion { get; set; }
        public string YourVersionString { get; set; }
        public string YourReleaseDate { get; set; }

        public Version LatestVersion { get; set; }
        public string LatestVersionString { get; set; }
        public string LatestReleaseDate { get; set; }

        public string ReasonsToUpdate { get; set; }

        private bool _hasCheckedVersion;

        public AutoUpdateDialog()
        {
            _hasCheckedVersion = false;
            InitializeComponent();
            IsUpdating = false;
            IsUpdateAvailable = false;
            Loaded += new RoutedEventHandler(StatusListDialogLoaded);
        }

        void StatusListDialogLoaded(object sender, RoutedEventArgs e)
        {
            // Only download the version info if it has not already been loaded
            if (!_hasCheckedVersion)
                UpdateVersionInfo();

            // Update the user interface
            Title = ProductName;

            YourReleaseCaption.Text = "Your version:";
            LatestReleaseCaption.Text = "Latest version:";
            ReasonsCaption.Text = "Reasons for update:";

            YourVersionField.Text = YourVersion.ToString();
            YourReleaseField.Text = YourReleaseDate;

            // Update the latest version info
            LatestVersionField.Text = LatestVersion != null ? LatestVersion.ToString() : "n/a";
            LatestReleaseField.Text = LatestReleaseDate ?? "n/a";

            string question = "";

            if (LatestVersion == null)
            {
                question = "Could not download information about the latest version.";
            }
            if (LatestVersion != null && YourVersion < LatestVersion)
            {
                question = "A new release of {0} is available. Do you want to update?";
            }
            if (LatestVersion != null && YourVersion == LatestVersion)
            {
                question = "You are using the latest official version.";
            }
            if (LatestVersion != null && YourVersion > LatestVersion)
            {
                question = "You have a new unofficial version.";
                UpdateButton.Content = "Download official";
            }

            // Update the question label
            UpdateQuestion.Text = String.Format(question, ProductName);

            // Only enable the updatebutton if an url is provided
            UpdateButton.IsEnabled = DownloadUrl != null;
            if (DownloadUrl != null)
                UpdateButton.ToolTip = String.Format("Download a new version from {0}", DownloadUrl);

            ReasonsField.Text = ReasonsToUpdate;
            ReasonsField.Visibility = ReasonsCaption.Visibility = IsUpdateAvailable ? Visibility.Visible : Visibility.Collapsed;
        }

        public void UpdateVersionInfo()
        {
            try
            {
                // Retrieve assembly data
                if (CurrentAssembly == null)
                    CurrentAssembly = Assembly.GetEntryAssembly();

                FileVersionInfo fvi = FileVersionInfo.GetVersionInfo(CurrentAssembly.Location);
                if (ProductName == null)
                    ProductName = fvi.ProductName;

                FileInfo fi = new FileInfo(CurrentAssembly.Location);
                YourVersion = CurrentAssembly.GetName().Version;
                YourVersionString = CurrentAssembly.GetName().Version.ToString();
                YourReleaseDate = fi.CreationTime.ToShortDateString();

                DownloadLatestVersionInfo();
            }
            catch (Exception e)
            {
                Trace.WriteLine(e.Message);
            }
            finally
            {
                _hasCheckedVersion = true;
            }
        }

        void DownloadLatestVersionInfo()
        {
            if (UpdateUrl == null) return;

            // Get last version
            XmlTextReader r = new XmlTextReader(UpdateUrl);
            XmlDocument x = new XmlDocument();
            try
            {
                x.Load(r);
                XmlNode releaseNode = x.SelectSingleNode("Product/Release");
                string latestVersion = GetNodeText(releaseNode, "@Version");
                LatestReleaseDate = GetNodeText(releaseNode, "@ReleaseDate");
                if (latestVersion != null)
                    LatestVersion = new Version(latestVersion);

                // Format the release date to current local format
                DateTime release;
                if (LatestReleaseDate != null && DateTime.TryParse(LatestReleaseDate, out release))
                    LatestReleaseDate = release.ToShortDateString();

                string reasonsToUpdate = GetNodeText(releaseNode, "ReasonsToUpdate");
                if (reasonsToUpdate != null)
                    ReasonsToUpdate = reasonsToUpdate;

                // Update url to download latest version
                string download = GetNodeText(releaseNode, "@Download");
                if (download != null)
                    DownloadUrl = download;
                IsUpdateAvailable = YourVersion < LatestVersion;
            }
            catch (WebException e)
            {
                TraceException(e);
            }
        }

        private static void TraceException(Exception e)
        {
            Exception ex = e;
            while (ex != null)
            {
                Trace.WriteLine(ex.Message);
                ex = ex.InnerException;
            }
        }

        private static string GetNodeText(XmlNode x, string xpath)
        {
            XmlNode n = x.SelectSingleNode(xpath);
            if (n == null) return null;
            return n.InnerText;
        }

        private void Update_Click(object sender, RoutedEventArgs e)
        {
            if (DownloadUrl != null)
            {
                Process.Start(DownloadUrl);
                IsUpdating = true;
                Close();
                Application.Current.Shutdown();
            }
        }

        private void DontUpdate_Click(object sender, RoutedEventArgs e)
        {
            Close();
        }

        /// <summary>
        /// Show the auto update dialog, update "CheckForProgramUpdates" in the ApplicationSettings
        /// </summary>
        /// <param name="settings"></param>
        /// <param name="url"></param>
        public static void Show(ApplicationSettingsBase settings, string url)
        {
            AutoUpdateDialog d = new AutoUpdateDialog();
            if (settings!=null)
                d.CheckForProgramUpdates = (bool)settings["CheckForProgramUpdates"];
            d.UpdateUrl = url;
            d.ShowDialog();
            if (settings!=null)
                settings["CheckForProgramUpdates"] = d.CheckForProgramUpdates;
        }

        /// <summary>
        /// Show the auto update dialog if an update is available
        /// </summary>
        /// <param name="settings"></param>
        /// <param name="updateUrl"></param>
        public static void ShowIfUpdateAvailable(ApplicationSettingsBase settings, string updateUrl)
        {
            AutoUpdateDialog d = new AutoUpdateDialog();
            if (settings != null)
                d.CheckForProgramUpdates = (bool)settings["CheckForProgramUpdates"];

            d.UpdateUrl = updateUrl;
            d.UpdateVersionInfo();
            if (d.IsUpdateAvailable)
                d.ShowDialog();
            
            if (settings != null)
                settings["CheckForProgramUpdates"] = d.CheckForProgramUpdates;
            d.Close();
        }
    }
}
