﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Input;
using System.Net;
using System.Xml;
using System.Diagnostics;
using System.Globalization;
using System.Reflection;
using System.Windows.Markup;
using System.Configuration;

namespace OpenSourceControls
{
    /// <summary>
    /// StatusList Dialog
    /// </summary>
    public partial class StatusDialog : Window
    {
        public bool IsNewStatusItemAvailable { get; set; }
        public string StatusListUrl { get; set; }
        public string ProductName { get; set; }
        public Assembly CurrentAssembly { get; set; }
        public DateTime LastReadDate { get; set; }

        public bool CheckStatusList
        {
            get { return (bool)GetValue(CheckStatusListProperty); }
            set { SetValue(CheckStatusListProperty, value); }
        }

        public static readonly DependencyProperty CheckStatusListProperty =
            DependencyProperty.Register("CheckStatusList", typeof(bool), typeof(AutoUpdateDialog), new UIPropertyMetadata(true));

        bool _hasCheckedStatusList;

        public StatusDialog()
        {
            _hasCheckedStatusList = false;
            InitializeComponent();
            IsNewStatusItemAvailable = false;
            Loaded += new RoutedEventHandler(StatusListDialogLoaded);
        }

        void StatusListDialogLoaded(object sender, RoutedEventArgs e)
        {
            if (!_hasCheckedStatusList)
                UpdateStatusList();

            LastReadDate = DateTime.Now;
        }

        public void UpdateStatusList()
        {
            try
            {
                // Retrieve assembly data
                if (CurrentAssembly == null)
                    CurrentAssembly = Assembly.GetEntryAssembly();

                // Get the name of the product
                FileVersionInfo fvi = FileVersionInfo.GetVersionInfo(CurrentAssembly.Location);
                if (ProductName == null)
                    ProductName = fvi.ProductName;
                Title = String.Format("{0} status list", ProductName);

                BindStatusList();
                StatusListGroup.ToolTip = String.Format("Last read {0}", LastReadDate.ToString());
            }
            catch (Exception e)
            {
                Trace.WriteLine(e.Message);
            }
            finally
            {
                _hasCheckedStatusList = true;
            }
        }

        private void BindStatusList()
        {
            if (StatusListUrl == null) return;
            // Update the status list provider

#if UseXmlDataProvider
                XmlDataProvider p = Resources["StatusListProvider"] as XmlDataProvider;
                try
                {
                    p.Source = new Uri(StatusListSource, UriKind.RelativeOrAbsolute);
                }
                catch (Exception e)
                {
                    Trace.WriteLine(e.Message);
                }
            }
#endif

            IsNewStatusItemAvailable = false;
            try
            {
                XmlTextReader r = new XmlTextReader(StatusListUrl);
                XmlDocument x = new XmlDocument();
                x.Load(r);

                XmlNodeList listNodes = x.SelectNodes("/rss/channel/item");
                List<XmlNode> copyOfNodes = new List<XmlNode>();
                if (listNodes == null)
                    return;

                foreach (XmlNode n in listNodes)
                {
                    XmlNode pubNode = n.SelectSingleNode("pubDate");
                    if (pubNode == null) continue;
                    DateTime dt = DateTime.Parse(pubNode.InnerText);
                    XmlAttribute newAttribute = x.CreateAttribute("isNew");
                    XmlAttribute textColorAttribute = x.CreateAttribute("textColor");
                    if (dt > LastReadDate)
                    {
                        newAttribute.Value = "Visible";
                        textColorAttribute.Value = "";
                        IsNewStatusItemAvailable = true;
                    }
                    else
                    {
                        newAttribute.Value = "Collapsed";
                        textColorAttribute.Value = "Gray";
                    }
                    n.Attributes.Append(newAttribute);
                    n.Attributes.Append(textColorAttribute);
                    copyOfNodes.Add(n);
                }

                // Sort items by date/time
                var sortedNodes = copyOfNodes.OrderByDescending(node => DateTime.Parse(node.SelectSingleNode("pubDate").InnerText)).ToList();
                StatusList.ItemsSource = sortedNodes;
            }
            catch (WebException e)
            {
                Trace.WriteLine(e.Message);
            }
        }

        private void Items_MouseDoubleClick(object sender, MouseButtonEventArgs e)
        {
            ListBox l = (ListBox)sender;
            XmlElement x = l.SelectedItem as XmlElement;
            if (x == null)
                return;
            string link = x["link"].InnerText;
            // Open the link
            if (link != null)
                Process.Start(link);
        }

        private void Ok_Click(object sender, RoutedEventArgs e)
        {
            Close();
        }

        /// <summary>
        /// Show the status list
        /// </summary>
        /// <param name="settings"></param>
        /// <param name="url"></param>
        public static void Show(ApplicationSettingsBase settings, string url)
        {
            StatusDialog d = new StatusDialog();
            if (settings != null)
            {
                d.CheckStatusList = (bool)settings["CheckStatusList"];
                d.LastReadDate = (DateTime)settings["LastStatusCheck"];
            }
            d.StatusListUrl = url;
            d.ShowDialog();
            if (settings != null)
            {
                settings["LastStatusCheck"] = d.LastReadDate;
                settings["CheckStatusList"] = d.CheckStatusList;
            }
        }

        /// <summary>
        /// Show the status list if it has been updated
        /// </summary>
        /// <param name="settings"></param>
        /// <param name="url"></param>
        public static void ShowIfUpdated(ApplicationSettingsBase settings, string url)
        {
            StatusDialog d = new StatusDialog();
            if (settings != null)
            {
                d.LastReadDate = (DateTime)settings["LastStatusCheck"];
                d.CheckStatusList = (bool)settings["CheckStatusList"];
            }
            d.StatusListUrl = url;
            d.UpdateStatusList();
            if (d.IsNewStatusItemAvailable)
                d.ShowDialog();
            if (settings != null)
            {
                settings["LastStatusCheck"] = d.LastReadDate;
                settings["CheckStatusList"] = d.CheckStatusList;
            }
            d.Close();
        }
    }

    // http://www.drwpf.com/Blog/Default.aspx?tabid=36&EntryID=48

    [ValueConversion(typeof(string), typeof(string))]
    public class DateConverter : MarkupExtension, IValueConverter
    {
        private static DateConverter _converter = null;
        public override object ProvideValue(IServiceProvider serviceProvider)
        {
            if (_converter == null)
            {
                _converter = new DateConverter();
            }
            return _converter;
        }

        #region IValueConverter Members

        public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            if (targetType != typeof(string)) return null;
            if (!(value is string)) return null;
            DateTime d = DateTime.Parse(value.ToString(), CultureInfo.InvariantCulture);
            return d.ToString("dd.MM.yyyy");
        }

        public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            throw new NotImplementedException();
        }

        #endregion
    }

    [ValueConversion(typeof(string), typeof(string))]
    public class HtmlConverter : MarkupExtension, IValueConverter
    {
        private static HtmlConverter _converter = null;
        public override object ProvideValue(IServiceProvider serviceProvider)
        {
            if (_converter == null)
            {
                _converter = new HtmlConverter();
            }
            return _converter;
        }

        #region IValueConverter Members

        public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            if (targetType != typeof(string)) return null;
            string src = value as string;
            if (src == null) return null;
            return src;
            /*string result = src.Replace("\r\n", "");
            result = result.Replace(@"<br \>", "\r\n");
            // result = result.Replace(@"<br \>", " ");
            //result = result.Replace(@"  ", " ");
            result = result.Trim("\r\n ".ToCharArray());
            return result;*/
        }

        public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            throw new NotImplementedException();
        }

        #endregion
    }

}
